<?php

// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * External Web Service Template
 *
 * @package    localwstemplate
 * @copyright  2011 Moodle Pty Ltd (http://moodle.com)
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
require_once($CFG->libdir . "/externallib.php");

class uoc_get_user_calendar_events_external extends external_api {

    /**
     * Returns description of method parameters
     * @return external_function_parameters
     */
    public static function uoc_get_user_calendar_events_parameters() {
        return new external_function_parameters(
                array('welcomemessage' => new external_value(PARAM_TEXT, 'The welcome message. By default it is "Hello world,"', VALUE_DEFAULT, 'Hello world, '))
        );
    }

    /**
     * Returns welcome message
     * @return string welcome message
     */                   
    public static function  uoc_get_user_calendar_events($welcomemessage = 'Hola NOIS, ') {
        global $USER;

        //Parameter validation
        //REQUIRED
        $params = self::validate_parameters(self:: uoc_get_user_calendar_events_parameters(),
                array('welcomemessage' => $welcomemessage));

        //Context validation
        //OPTIONAL but in most web service it should present
        $context = get_context_instance(CONTEXT_USER, $USER->id);
        self::validate_context($context);

        //Capability checking
        //OPTIONAL but in most web service it should present
        if (!has_capability('moodle/user:viewdetails', $context)) {
            throw new moodle_exception('cannotviewprofile');
        }


        $eventinfo = array();
            $eventinfo['id'] = 3;
            $eventinfo['name'] = 'Entrega PAC3';
            $eventinfo['description'] = 'Entrega PAC3 del PFC';
            $eventinfo['format'] =1;
            $eventinfo['userid'] = 2;
            $eventinfo['eventype'] = 'Entrega';
            $eventinfo['startdate']='201212271500';
            $eventinfo['endate']='201212271500';
            $eventsinfo=array($eventinfo);
            return $eventinfo;
    }
    

    /**
     * Returns description of method result value
     * @return external_description
     */
    public static function  uoc_get_user_calendar_events_returns() {
    return new  external_single_structure(
                        array(
                            'id' => new external_value(PARAM_INT, 'Notification'),
                            'name' => new external_value(PARAM_TEXT,'small message,..'),
                            'description' => new external_value(PARAM_TEXT, 'full message', VALUE_OPTIONAL),
                            'format' => new external_value(PARAM_INT, 'number of weeks/topics'),
                            'userid' => new external_value(PARAM_INT,'number of recent items appearing on the course page', VALUE_OPTIONAL),
                            'eventype' => new external_value(PARAM_TEXT,'timestamp when the course start'), 
                            'startdate' => new external_value(PARAM_TEXT,'timestamp when the course start'), 
                            'endate' => new external_value(PARAM_TEXT,'timestamp when the course start'),                            
                     ), 'events'
                
        );
    }


}
