<?php

// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * External Web Service Template
 *
 * @package    localwstemplate
 * @copyright  2011 Moodle Pty Ltd (http://moodle.com)
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
require_once($CFG->libdir . "/externallib.php");

class uoc_get_user_unread_messages_external extends external_api {

    /**
     * Returns description of method parameters
     * @return external_function_parameters
     */
    public static function uoc_get_user_unread_messages_parameters() {
        return new external_function_parameters(
              	array('usr' => new external_value(PARAM_INT, 'id of the sender user'))
        );
    }

    /**
     * Returns welcome message
     * @return string welcome message
     */                   
    public static function uoc_get_user_unread_messages($welcomemessage = 'Hola NOIS, ') {
 		 global $CFG, $USER, $DB;
 		require_once($CFG->dirroot . "/message/lib.php");
        //check if messaging is enabled
        if (!$CFG->messaging) {
            throw new moodle_exception('disabled', 'message');
        }

        // Ensure the current user is allowed to run this function
        $context = get_context_instance(CONTEXT_SYSTEM);
        self::validate_context($context);
        require_capability('moodle/site:sendmessage', $context);
        
       $params = self::validate_parameters(self::uoc_get_user_unread_messages_parameters(), array('usr' => $usr));

        //retrieve count of unread messages
       $notifications = message_get_unread_conversations($USER,0,100);
       $notificationsinfo = array();
		foreach ($notifications as $notification) {
		
		    $notificationinfo = array();
            $notificationinfo['message_read_id'] = $notification->message_read_id;
            $notificationinfo['subject'] = $notification->subject;
            $notificationinfo['smallmessage'] = $notification->smallmessage;
            $notificationinfo['fullmessage'] = $notification->fullmessage;
            $notificationinfo['timecreated'] = $notification->timecreated;
            $notificationinfo['useridfrom'] = $notification->useridfrom;
            $notificationinfo['useridto'] = $notification->useridto;
            $notificationinfo['contexturl'] = $notification->contexturl;
            $notificationinfo['contexturlname'] = $notification->contexturlname;
            $notificationsinfo[] = $notificationinfo;
		}
		
        return $notificationsinfo;
       
    
    }

    /**
     * Returns description of method result value
     * @return external_description
     */
    public static function uoc_get_user_unread_messages_returns() {
       return new external_multiple_structure(
                new external_single_structure(
                        array(
                            'message_read_id' => new external_value(PARAM_INT, 'Notification'),
                            'subject' => new external_value(PARAM_TEXT,'subject,..'),
                            'smallmessage' => new external_value(PARAM_TEXT,'small message,..'),
                            'fullmessage' => new external_value(PARAM_TEXT, 'full message', VALUE_OPTIONAL),
                            'timecreated' => new external_value(PARAM_INT, 'number of weeks/topics'),
                            'useridfrom' => new external_value(PARAM_INT, 'userfromid'),
                            'useridto' => new external_value(PARAM_INT, 'usertoid'),
                            'contexturl' => new external_value(PARAM_TEXT,'number of recent items appearing on the course page', VALUE_OPTIONAL),
                            'contexturlname' => new external_value(PARAM_TEXT,'timestamp when the course start'),                            
                     ), 'notifications'
                )
        );
    }



}
